import React, { useState } from 'react';
import { View, Image, Linking, Text, Switch, Platform } from 'react-native';
import { CommonActions, useNavigation } from '@react-navigation/native';
import * as Notifications from 'expo-notifications';

import { WarningModal } from '../WarningModal';
import { MenuButton } from '../MenuButton';
import { styles } from './styles';
import { StoreType, storage } from 'src/storage';
import { useDeleteUserMutation } from '@api/app';
import { Colors } from 'src/theme';
import { NAVIGATION_PAGES } from 'src/types';

import MailIcon from '../../../assets/icons/mail.svg';
import DocumentIcon from '../../../assets/icons/document.svg';
import ExitIcon from '../../../assets/icons/exit.svg';
import UserXMark from '../../../assets/icons/user-xmark.svg';
import InfoIcon from 'assets/icons/info-solid.svg';

import { APP_VERSION, FASTEST_MAP_HOST } from 'src/constants';
import { useNotification } from 'src/contexts/NotificationContext';
import { usePostSaveNotificationTokenMutation } from '@api/user';
import { usePushNotification } from 'src/contexts/PushNotificationContext';

export const MenuDrawer = (props: any) => {
  const { mutate: deleteUser } = useDeleteUserMutation();
  const token = storage.get('token', StoreType.STRING) as string;
  const navigation = useNavigation();
  const [modalInfo, setModalInfo] = useState({
    visible: false,
    type: 'confirm',
    message: '',
    action: () => {}
  });
  const { updateNotificationStatus } = useNotification();
  const { mutateAsync: saveNotificationToken } = usePostSaveNotificationTokenMutation();
  const [shouldOpenWarningModal, setShouldOpenWarningModal] = useState(false);
  const { isSubscribed, toggleSubscription } = usePushNotification();

  const openModal = (type: string, message: string, action: any) => {
    setModalInfo({
      visible: true,
      type,
      message,
      action
    });
  };

  const closeModal = () => {
    setModalInfo({ ...modalInfo, visible: false });
  };

  const handleLogout = () => {
    storage.remove('token');
    storage.remove('uid');
    storage.remove('currentUserData');
    storage.remove('visitedTilesUrl');
    storage.remove('filterSettings');
    updateNotificationStatus();
    navigation.dispatch(
      CommonActions.reset({
        index: 1,
        routes: [{ name: NAVIGATION_PAGES.WELCOME }]
      })
    );
  };

  const handleDeleteAccount = () => {
    deleteUser({ token }, { onSuccess: handleLogout });
  };

  const handleSubscribe = async () => {
    const deviceData = await registerForPushNotificationsAsync();

    if (deviceData?.notificationToken) {
      toggleSubscription();
      await saveNotificationToken({
        token,
        platform: deviceData.platform,
        n_token: deviceData.notificationToken
      });
    }
  };

  const toggleSwitch = async () => {
    if (isSubscribed) {
      toggleSubscription();
    } else {
      const { status } = await Notifications.getPermissionsAsync();
      if (status !== 'granted') {
        setModalInfo({
          visible: true,
          type: 'success',
          message:
            'To use this feature we need your permission to access your notifications. If you press OK your system will ask you to confirm permission to receive notifications from NomadMania.',
          action: () => setShouldOpenWarningModal(true)
        });
      } else {
        handleSubscribe();
      }
    }
  };

  async function registerForPushNotificationsAsync() {
    const { status: existingStatus } = await Notifications.getPermissionsAsync();
    let finalStatus = existingStatus;
    if (existingStatus !== 'granted') {
      const { status } = await Notifications.requestPermissionsAsync();
      finalStatus = status;
    }
    if (finalStatus !== 'granted') {
      setModalInfo({
        visible: true,
        type: 'success',
        message:
          'NomadMania app needs notification permissions to function properly. Open settings?',
        action: () =>
          Platform.OS === 'ios' ? Linking.openURL('app-settings:') : Linking.openSettings()
      });
      return null;
    }
    const deviceData = await Notifications.getDevicePushTokenAsync();
    console.log('deviceData', deviceData);

    if (Platform.OS === 'android') {
      Notifications.setNotificationChannelAsync('default', {
        name: 'default',
        importance: Notifications.AndroidImportance.MAX,
        vibrationPattern: [0, 250, 250, 250],
        lightColor: '#FF231F7C'
      });
    }
    storage.set('deviceToken', deviceData.data);

    return { notificationToken: deviceData.data ?? '', platform: deviceData.type ?? '' };
  }

  return (
    <>
      <View style={styles.container}>
        <View style={{ flex: 1 }}>
          <View style={styles.logoContainer}>
            <Image source={require('../../../assets/logo-ua.png')} style={styles.logo} />
          </View>
          <MenuButton
            label="Info"
            icon={<InfoIcon fill={Colors.DARK_BLUE} width={20} height={20} />}
            red={false}
            buttonFn={() => navigation.navigate(NAVIGATION_PAGES.INFO as never)}
          />
          <MenuButton
            label="Contact Us"
            icon={<MailIcon fill={Colors.DARK_BLUE} width={20} height={20} />}
            red={false}
            buttonFn={() => Linking.openURL('https://nomadmania.com/contact/')}
          />
          <MenuButton
            label="Terms & Conditions"
            icon={<DocumentIcon fill={Colors.DARK_BLUE} width={20} height={20} />}
            red={false}
            buttonFn={() => Linking.openURL('https://nomadmania.com/terms/')}
          />
          <View
            style={{
              display: 'flex',
              flexDirection: 'row',
              justifyContent: 'space-between',
              marginTop: 20,
              alignItems: 'center'
            }}
          >
            <Text style={{ color: Colors.DARK_BLUE, fontSize: 16, fontWeight: 'bold' }}>
              Notifications
            </Text>
            <Switch
              trackColor={{ false: Colors.LIGHT_GRAY, true: Colors.DARK_BLUE }}
              thumbColor={Colors.WHITE}
              onValueChange={toggleSwitch}
              value={isSubscribed}
              style={{ transform: 'scale(0.8)' }}
            />
          </View>
        </View>

        <View style={styles.bottomMenu}>
          {token ? (
            <>
              <MenuButton
                label="Logout"
                icon={<ExitIcon fill={Colors.RED} width={20} height={20} />}
                red={true}
                buttonFn={() =>
                  openModal('confirm', 'Are you sure you want to logout?', handleLogout)
                }
              />
              <MenuButton
                label="Delete account"
                icon={<UserXMark fill={Colors.RED} width={20} height={20} />}
                red={true}
                buttonFn={() =>
                  openModal(
                    'confirm',
                    'Are you sure you want to delete your account?',
                    handleDeleteAccount
                  )
                }
              />
            </>
          ) : null}

          <View style={{ gap: 6, marginTop: 16 }}>
            <Text style={styles.bottomText}>Version {APP_VERSION}</Text>
            <Text style={styles.bottomText}>
              Map server:{'\n'}
              {FASTEST_MAP_HOST}
            </Text>
          </View>
        </View>
      </View>

      <WarningModal
        isVisible={modalInfo.visible}
        onClose={closeModal}
        type={modalInfo.type}
        message={modalInfo.message}
        action={() => {
          modalInfo.action();
          closeModal();
        }}
        onModalHide={() => {
          if (shouldOpenWarningModal) {
            setShouldOpenWarningModal(false);
            handleSubscribe();
          }
        }}
      />
    </>
  );
};
