import React, { useEffect, useState, useContext, createContext } from 'react';
import * as Notifications from 'expo-notifications';
import { storage, StoreType } from 'src/storage';
import { Linking, Platform } from 'react-native';
import { useNavigation } from '@react-navigation/native';

const PushNotificationContext = createContext(null);

export const usePushNotification = () => useContext(PushNotificationContext);

export const PushNotificationProvider = ({ children }) => {
  const [isSubscribed, setIsSubscribed] = useState(
    (storage.get('subscribed', StoreType.BOOLEAN) as boolean) ?? false
  );
  const navigation = useNavigation();

  const lastNotificationResponse = Notifications.useLastNotificationResponse();

  useEffect(() => {
    if (lastNotificationResponse && Platform.OS === 'android') {
      console.log(
        'lastNotificationResponse',
        lastNotificationResponse.notification.request.content.data
      );
      const data = lastNotificationResponse.notification.request.content.data;

      if (data?.screen && data?.parentScreen) {
        if (data?.params) {
          navigation.navigate(
            ...([
              data.parentScreen,
              {
                screen: data.screen,
                params: data.params
              }
            ] as never)
          );
        } else {
          navigation.navigate(
            ...([
              data.parentScreen,
              {
                screen: data.screen
              }
            ] as never)
          );
        }
      }
      if (data?.url) {
        Linking.openURL(data.url);
      }
    }
  }, [lastNotificationResponse]);

  useEffect(() => {
    if (isSubscribed) {
      const notificationListener = Notifications.addNotificationReceivedListener((notification) => {
        console.log('Notification received', notification.request);
      });

      const responseListener = Notifications.addNotificationResponseReceivedListener((response) => {
        console.log('Notification response received', response.notification.request);

        let screenName;
        let url;
        let parentScreen;
        let params;
        if (Platform.OS === 'ios') {
          console.log('data ios', response.notification.request.trigger?.payload);
          parentScreen = response.notification.request.trigger?.payload?.parentScreen;
          screenName = response.notification.request.trigger?.payload?.screen;
          params = response.notification.request.trigger?.payload?.params;

          url = response.notification.request.trigger?.payload?.url;
        }

        if (screenName && parentScreen) {
          if (params) {
            navigation.navigate(
              ...([
                parentScreen,
                {
                  screen: screenName,
                  params: params
                }
              ] as never)
            );
          } else {
            navigation.navigate(
              ...([
                parentScreen,
                {
                  screen: screenName
                }
              ] as never)
            );
          }
        }
        if (url) {
          Linking.openURL(url);
        }
      });

      return () => {
        notificationListener.remove();
        responseListener.remove();
      };
    }
  }, [isSubscribed]);

  const subscribeToNotifications = async () => {
    storage.set('subscribed', true);
    setIsSubscribed(true);
  };

  const unsubscribeFromNotifications = async () => {
    await removeNotificationTokenFromServer();
    storage.remove('deviceToken');
    storage.set('subscribed', false);
    setIsSubscribed(false);
  };

  const toggleSubscription = async () => {
    if (isSubscribed) {
      await unsubscribeFromNotifications();
    } else {
      await subscribeToNotifications();
    }
  };

  return (
    <PushNotificationContext.Provider value={{ isSubscribed, toggleSubscription }}>
      {children}
    </PushNotificationContext.Provider>
  );
};

async function removeNotificationTokenFromServer() {}
